from datetime import datetime
from typing import Any, List, Literal, Optional, Tuple, Union
from uuid import UUID

from .base import AbstractApi


class OtmV2(AbstractApi):
    """
    Client interface to the OTM API located at
    https://api.satellitevu.com/otm/v2/docs.
    """

    api_path = "otm/v2"
    scopes = []

    def post_feasibility(
        self,
        *,
        contract_id: Union[UUID, str],
        coordinates: Union[Tuple[float, float], Tuple[float, float, float]],
        date_from: datetime,
        date_to: datetime,
        day_night_mode: Literal["day", "night", "day-night"] = "day-night",
        product: Literal["standard", "assured"] = "standard",
        max_cloud_cover: Optional[int] = None,
        min_off_nadir: Optional[int] = None,
        max_off_nadir: Optional[int] = None,
        min_gsd: Optional[float] = None,
        max_gsd: Optional[float] = None,
        **kwargs,
    ):
        """
        Creates a tasking feasibility request.

        Args:
            contract_id: Associated ID of the Contract under which the feasibility
            request will be performed.

            coordinates: An array of coordinates - (longitude, latitude) or
            (longitude, latitude, altitude).

            date_from: datetime representing the start date of the feasibility request.

            date_to: datetime representing the end date of the feasibility request.

            day_night_mode: String representing the mode of data capture. Allowed
            values are ["day", "night", "day-night"]. Defaults to "day-night".

            product: String representing a tasking option. Selecting "assured"
            allows visibility of all passes within the datetime interval. The
            user must accept all cloud cover risk.

            max_cloud_cover: Optional integer representing the maximum threshold
            of acceptable cloud coverage. Measured in percent. Defaults to 100.

            min_off_nadir: Optional integer representing the minimum angle from
            the sensor between nadir and the scene center. Measured in decimal
            degrees. Defaults to 0.

            max_off_nadir: Optional integer representing the maximum angle from
            the sensor between nadir and the scene center. Measured in decimal
            degrees. Must be larger than min_off_nadir. Defaults to 45.

            min_gsd: Optional float representing the minimum ground sample
            distance value. Measured in metres, this value reflects the square
            root of the area of the pixel size projected onto the earth. Defaults
            to 3.5.

            max_gsd: Optional float representing the minimum ground sample
            distance value. Measured in metres, this value reflects the square
            root of the area of the pixel size projected onto the earth. Defaults
            to 6.8.

            Please note that min/max off nadir and min/max gsd are mutually exclusive.
            You must pick either the off nadir angle or gsd as parameters.

        Kwargs:
            Allows sending additional parameters that are supported by the API but not
            added to this SDK yet.

        Returns:
            A dictionary containing properties of the feasibility request.
        """
        url = self.url(f"{str(contract_id)}/tasking/feasibilities/")
        payload = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": coordinates,
            },
            "properties": {
                "datetime": f"{date_from.isoformat()}/{date_to.isoformat()}",
                "product": product,
                **kwargs,
            },
        }

        if product == "standard":
            payload["properties"].update(
                {
                    "satvu:day_night_mode": day_night_mode,
                    "max_cloud_cover": max_cloud_cover,
                    "min_off_nadir": min_off_nadir,
                    "max_off_nadir": max_off_nadir,
                    "min_gsd": min_gsd,
                    "max_gsd": max_gsd,
                }
            )

        response = self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )

        if response.status != 202:
            raise Exception(f"Error - {response.status} : {response.text}")

        return response.json()

    def get_feasibility(self, *, contract_id: Union[UUID, str], id: Union[UUID, str]):
        """
        Retrieve the feasibility request with a given ID.

        Args:
            contract_id: Associated ID of the Contract for the given feasibility
            request.

            id: UUID representing the feasibility request e.g.
            "2009466e-cccc-4712-a489-b09aeb772296".

        Returns:
            A dictionary containing properties of the feasibility request.
        """
        response = self.make_request(
            method="GET",
            url=self.url(f"{str(contract_id)}/tasking/feasibilities/{str(id)}"),
        )
        return response.json()

    def get_feasibility_response(
        self, *, contract_id: Union[UUID, str], id: Union[UUID, str]
    ):
        """
        Retrieves the feasibility response for the feasibility request
        with a given id.

        Args:
            contract_id: Associated ID of the Contract for the given feasibility
            request.

            id: UUID representing the feasibility request e.g.
            "2009466e-cccc-4712-a489-b09aeb772296".

        Returns:
            A dictionary containing the feasibility response.
        """
        response = self.make_request(
            method="GET",
            url=self.url(
                f"{str(contract_id)}/tasking/feasibilities/{str(id)}/response"
            ),
        )
        return response.json()

    def list_feasibility_requests(
        self,
        *,
        contract_id: Union[UUID, str],
        per_page: int = 25,
        page_token: Optional[str] = None,
    ):
        """
        Retrieves a list of your feasibility requests.

        Args:
            contract_id: Associated ID of the Contract for which all feasibility
            requests will be listed.

            per_page: Number of results (defaults to 25) to be returned per page.

            page_token: Optional string key used to return specific page of results.
            Defaults to None -> assumes page 0.

        Returns:
            A dictionary containing a list of feasibility requests and their properties.
        """
        query = f"per_page={per_page}"
        if page_token:
            query += f"&token={page_token}"

        response = self.make_request(
            method="GET",
            url=self.url(f"{str(contract_id)}/tasking/feasibilities/?{query}"),
        )
        return response.json()

    def create_order(
        self,
        *,
        contract_id: Union[UUID, str],
        coordinates: Union[Tuple[float, float], Tuple[float, float, float]],
        date_from: datetime,
        date_to: datetime,
        day_night_mode: Literal["day", "night", "day-night"] = "day-night",
        product: Literal["standard", "assured"] = "standard",
        max_cloud_cover: Optional[int] = None,
        min_off_nadir: Optional[int] = None,
        max_off_nadir: Optional[int] = None,
        min_gsd: Optional[float] = None,
        max_gsd: Optional[float] = None,
        signature: Optional[str] = None,
        **kwargs,
    ):
        """
        Creates a tasking order request.

        Args:
            contract_id: Associated ID of the Contract under which the tasking
            order will be submitted.

            coordinates: An array of coordinates - (longitude, latitude) or
            (longitude, latitude, altitude).

            date_from: datetime representing the start date of the order.

            date_to: datetime representing the end date of the order.

            day_night_mode: String representing the mode of data capture. Allowed
            values are ["day", "night", "day-night"]. Defaults to "day-night".

            product: String representing a tasking option. Selecting "assured"
            allows visibility of all passes within the datetime interval. The
            user must accept all cloud cover risk.

            signature: String representing a signature token required for orders
            with assured priority. Defaults to None.

            max_cloud_cover: Optional integer representing the maximum threshold
            of acceptable cloud coverage. Measured in percent. Defaults to 100.

            min_off_nadir: Optional integer representing the minimum angle from
            the sensor between nadir and the scene center. Measured in decimal
            degrees. Defaults to 0.

            max_off_nadir: Optional integer representing the maximum angle from
            the sensor between nadir and the scene center. Measured in decimal
            degrees. Must be larger than min_off_nadir. Defaults to 45.

            min_gsd: Optional integer representing the minimum ground sample
            distance value. Measured in metres, this value reflects the square
            root of the area of the pixel size projected onto the earth. Defaults
            to 3.5.

            max_gsd: Optional integer representing the minimum ground sample
            distance value. Measured in metres, this value reflects the square
            root of the area of the pixel size projected onto the earth. Defaults
            to 6.8.

            Please note that min/max off nadir and min/max gsd are mutually exclusive.
            You must pick either the off nadir angle or gsd as parameters.

        Kwargs:
            Allows sending additional parameters that are supported by the API but not
            added to this SDK yet.

        Returns:
            A dictionary containing properties of the order created.
        """
        url = self.url(f"{str(contract_id)}/tasking/orders/")
        payload = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": coordinates,
            },
            "properties": {
                "datetime": f"{date_from.isoformat()}/{date_to.isoformat()}",
                "product": product,
                **kwargs,
            },
        }

        if product == "assured":
            if not signature:
                raise Exception(
                    "Orders with assured priority must also have a signature token."
                )
            payload["properties"].update({"signature": signature})

        else:
            payload["properties"].update(
                {
                    "satvu:day_night_mode": day_night_mode,
                    "max_cloud_cover": max_cloud_cover,
                    "min_off_nadir": min_off_nadir,
                    "max_off_nadir": max_off_nadir,
                    "min_gsd": min_gsd,
                    "max_gsd": max_gsd,
                }
            )

        response = self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )

        if response.status != 201:
            raise Exception(f"Error - {response.status} : {response.text}")

        return response.json()

    def get_order(self, *, contract_id: Union[UUID, str], order_id: Union[UUID, str]):
        """
        Retrieve the order with a given order_id.

        Args:
            contract_id: Associated ID of the Contract under which the tasking
            order is stored.

            order_id: UUID representing the order id e.g.
            "2009466e-cccc-4712-a489-b09aeb772296".

        Returns:
            A dictionary containing properties of the order.
        """
        response = self.make_request(
            method="GET",
            url=self.url(f"{str(contract_id)}/tasking/orders/{str(order_id)}"),
        )
        return response.json()

    def list_orders(
        self,
        *,
        contract_id: Union[UUID, str],
        per_page: int = 25,
        page_token: Optional[str] = None,
    ):
        """
        Retrieves a list of your orders. The orders are returned sorted by creation
        date, with the most recent orders appearing first.

        Args:
            contract_id: Associated ID of the Contract under which all tasking
            orders will be listed.

            per_page: Number of results (defaults to 25) to be returned per page.

            page_token: Optional string key used to return specific page of results.
            Defaults to None -> assumes page 0.

        Returns:
            A dictionary containing a list of orders and their properties.
        """
        query = f"per_page={per_page}"
        if page_token:
            query += f"&token={page_token}"

        response = self.make_request(
            method="GET", url=self.url(f"{str(contract_id)}/tasking/orders/?{query}")
        )
        return response.json()

    def get_price(
        self,
        *,
        contract_id: Union[UUID, str],
        coordinates: Union[Tuple[float, float], Tuple[float, float, float]],
        date_from: datetime,
        date_to: datetime,
    ):
        """
        Returns the price for a set of ordering parameters.

        Args:
            contract_id: Associated ID of the Contract for which the pricebook
            will be listed.

            coordinates: An array of coordinates.

            date_from: datetime representing the start date of the order.

            date_to: datetime representing the end date of the order.

        Returns:
            A dictionary containing keys: price, created_at where the price field
            indicates the price, in tokens, for the set of ordering parameters and
            created_at is the UTC datetime at which the price was calculated.

        """
        url = self.url(f"{str(contract_id)}/tasking/price/")
        payload = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": coordinates,
            },
            "properties": {
                "datetime": f"{date_from.isoformat()}/{date_to.isoformat()}"
            },
        }

        response = self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )
        return response.json()

    def search(
        self,
        contract_id: Union[str, UUID],
        page_token: Optional[str] = None,
        per_page: int = 25,
        collections: Optional[List[str]] = None,
        ids: Optional[List[str]] = None,
        date_range: Optional[str] = None,
        created_at: Optional[str] = None,
        updated_at: Optional[str] = None,
        properties: Optional[dict] = None,
        intersects: Optional[Any] = None,
        sort_by: Optional[List[dict]] = None,
    ):
        """
        Performs a search across all feasibility and orders associated with a
        contract.

        Args:
            contract_id: Associated ID of the Contract under which the search
            request will be performed.

            page_token: Optional string key used to return specific page of results.
            Defaults to None -> assumes page 0.

            per_page: Number of results (defaults to 25) to be returned per page.

            collections: List of strings specifying the collections within which
            to search. Allowed values are:
            [ "feasibility", "feasibility|response|%", "orders", "otm|orders|%" ]

            ids: List of strings specifying the ids within which
            to search.

            date_range: Optional string specifying a closed datetime range within
            which to search e.g. "2023-03-22T12:50:24+01:00/2023-03-29T12:50:24+01:00"
            Assumed to be UTC if timezone offset is not given. This datetime range
            corresponds directly to the datetime property associated with the
            order/feasibility request/response item.

            created_at: Optional string specifying a closed datetime range within
            which to search e.g. "2023-03-22T12:50:24+01:00/2023-03-29T12:50:24+01:00"
            Assumed to be UTC if timezone offset is not given. This datetime range
            describes when the database entries are created.

            updated_at: Optional string specifying a closed datetime range within
            which to search e.g. "2023-03-22T12:50:24+01:00/2023-03-29T12:50:24+01:00"
            Assumed to be UTC if timezone offset is not given. This datetime range
            describes when database entries are updated.

            properties: Optional dictionary specifying the filterable fields and
            value. Fields can be filtered by 'status', 'min_off_nadir',
            'max_off_nadir', 'min_gsd', 'max_gsd'.
            Examples: {"status" : "failed", "max_gsd" : 6.5, "min_off_nadir": 35}

            intersects: Optional dictionary with keys "coordinates" and "geometry"
            type that search results intersect with. Available geometry types include:
            "Point","MultiPoint", "LineString", "MultiLineString", "Polygon",
            "MultiPolygon". For example:
            intersects = {"coordinates":[-1.065151, 51.163899], "type" : "Point"}.

            sort_by: List of parameters specifying the field and direction
            ('asc', 'desc') the results are sorted by. Currently only the 'status'
            field is sortable e.g. [{"field": "status", "direction": "desc"}].
        """
        url = self.url(f"{str(contract_id)}/search/")
        payload = {
            "token": page_token,
            "limit": per_page,
            "collections": collections,
            "ids": ids,
            "datetime": date_range,
            "created_at": created_at,
            "updated_at": updated_at,
            "properties": properties,
            "intersects": intersects,
            "sort_by": sort_by,
        }

        response = self.make_request(
            method="POST", url=url, json={k: v for k, v in payload.items() if v}
        )
        return response.json()



from abc import ABC, abstractmethod, abstractproperty
from importlib.metadata import version
from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Optional

if TYPE_CHECKING:
    from satellitevu import Auth


class ResponseWrapper(ABC):
    raw: Any
    headers: Dict[str, str]
    status: int
    json: Any
    text: str

    @abstractmethod
    def json(self):
        pass


class AbstractClient(ABC):
    """
    Abstract HTTP client
    """

    _auth: Dict[str, "Auth"]

    def __init__(self):
        self._auth = {}

    def post(
        self,
        url,
        *,
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Any] = None,
    ) -> ResponseWrapper:
        return self.request("POST", url, headers=headers, data=data, json=json)

    @abstractmethod
    def request(
        self,
        method: str,
        url: str,
        *,
        scopes: Iterable[str] = [],
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Any] = None,
    ) -> ResponseWrapper:
        pass

    def set_auth(self, base_url: str, auth):
        self._auth[base_url] = auth

    def _set_auth(
        self, url: str, headers: Mapping[str, str], scopes: Iterable[str] = []
    ):
        auth = next((v for k, v in self._auth.items() if url.startswith(k)), None)
        has_auth = next(
            (True for k in headers.keys() if k.lower() == "authorization"), False
        )
        if auth and not has_auth:
            headers["Authorization"] = f"Bearer {auth.token(scopes)}"

    def prepare_headers(
        self,
        url: str,
        headers: Mapping[str, str],
        scopes: Iterable[str] = [],
    ):
        _headers = {**(headers or {})}

        self._set_auth(url, _headers, scopes)

        sv_comment = f"(satellitevu/{version('satellitevu')})"
        _headers["User-Agent"] = f"{self.user_agent} {sv_comment}"

        return _headers

    @abstractproperty
    def user_agent(self) -> str:
        pass



from os import getenv as _getenv
from typing import Optional, Union

env_prefix = "SATELLITEVU_"


def getenv(key: str, default: Optional[str] = None) -> Union[str, None]:
    """
    Get env var for given key with defined prefix
    """
    return _getenv(f"{env_prefix}{key.upper()}", default)


def boolize(value: str) -> bool:
    """
    Convert string value to meaningful boolean value
    """
    return (value or "").lower() not in ["", "0", "false", "no", "n"]


def fqdn(subdomain, use_qa=False):
    """
    Build FQDN for given subdomain and `use_qa` setting
    """
    return ".".join(
        filter(lambda x: x, [subdomain, "qa" if use_qa else None, "satellitevu.com"])
    )


"""
Set SATELLITEVU_USE_QA to a truthy value to use production endpoints by default.
Individual config values can still be overridden with their env variables.
"""
USE_QA = boolize(getenv("USE_QA", "0"))

"""
Set SATELLITEVU_API_GATEWAY to an base URL to use for all API calls. Defaults to
https://api.satellitevu.com/
"""
GATEWAY = getenv("API_GATEWAY", f"https://{fqdn('api', USE_QA)}/")

"""
Set SATELLITEVU_API_AUDIENCE to override the requested audience for tokens.
"""
AUDIENCE = getenv("API_AUDIENCE", GATEWAY)

"""
Set SATELLITEVU_AUTH_URL to override the base URL for authentication calls.
"""
AUTH_URL = getenv("AUTH_URL", f"https://{fqdn('auth', USE_QA)}/")



from json import dumps, loads
from secrets import token_urlsafe
from urllib.parse import urlparse
from uuid import uuid4

from mocket import Mocket, mocketize
from mocket.mockhttp import Entry
from pytest import mark, raises

API_PATH_FEASIBILITY = "otm/v2/contract-id/tasking/feasibilities/"
API_PATH_ORDERS = "otm/v2/contract-id/tasking/orders/"


@mocketize(strict_mode=True)
def test_cannot_use_v2_without_contract_id(client):
    error_message = "missing 1 required keyword-only argument: 'contract_id'"
    with raises(TypeError) as exc:
        client.otm_v2.get_order(order_id=uuid4())

    assert error_message in str(exc.value)


@mocketize(strict_mode=True)
@mark.parametrize("product", ("standard", "assured"))
def test_post_feasibility(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
    product,
):
    contract_id = otm_request_parameters["contract_id"]
    api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

    Entry.single_register(
        "POST",
        client._gateway_url + api_path,
        body=dumps(otm_response),
        status=202,
    )

    response = client.otm_v2.post_feasibility(
        **otm_request_parameters,
        product=product,
    )
    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"

    api_request_body = loads(api_request.body)
    assert api_request_body["geometry"] == {
        "type": "Point",
        "coordinates": otm_request_parameters["coordinates"],
    }

    properties_keys = ["max_cloud_cover", "min_off_nadir", "max_off_nadir"]

    if product == "assured":
        for key in properties_keys:
            assert not api_request_body["properties"].get(key)
    else:
        for key in properties_keys:
            assert api_request_body["properties"][key] == otm_request_parameters[key]

    assert (
        api_request_body["properties"]["datetime"]
        == f"{otm_request_parameters['date_from'].isoformat()}/{otm_request_parameters['date_to'].isoformat()}"  # noqa: E501
    )


@mocketize(strict_mode=True)
def test_get_feasibility(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
):
    feasibility_id = uuid4()
    contract_id = otm_request_parameters["contract_id"]
    api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

    Entry.single_register(
        "GET",
        client._gateway_url + api_path + str(feasibility_id),
        status=200,
        body=dumps(otm_response),
    )

    response = client.otm_v2.get_feasibility(contract_id=contract_id, id=feasibility_id)

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path + f"{feasibility_id}"
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
def test_get_feasibility_response(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
):
    feasibility_id = uuid4()
    contract_id = otm_request_parameters["contract_id"]
    api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

    Entry.single_register(
        "GET",
        client._gateway_url + api_path + str(feasibility_id) + "/response",
        status=200,
        body=dumps(otm_response),
    )

    response = client.otm_v2.get_feasibility_response(
        contract_id=contract_id, id=feasibility_id
    )

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path + f"{feasibility_id}/response"
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
@mark.parametrize(
    "per_page",
    (None, 10),
)
def test_list_feasibilities(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
    per_page,
):
    contract_id = otm_request_parameters["contract_id"]
    per_page = per_page if not None else 25
    api_path = (
        API_PATH_FEASIBILITY.replace("contract-id", str(contract_id)) + f"?{per_page=}"
    )

    Entry.single_register(
        "GET",
        client._gateway_url + api_path,
        status=200,
        body=dumps(otm_response),
    )

    response = client.otm_v2.list_feasibility_requests(
        contract_id=contract_id, per_page=per_page
    )

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
@mark.parametrize("product", ("standard", "assured"))
def test_post_order(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
    product,
):
    contract_id = otm_request_parameters["contract_id"]
    api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

    otm_request_parameters["product"] = product
    otm_request_parameters["signature"] = token_urlsafe(16)

    Entry.single_register(
        "POST",
        client._gateway_url + api_path,
        body=dumps(otm_response),
        status=201,
    )

    response = client.otm_v2.create_order(**otm_request_parameters)
    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"

    api_request_body = loads(api_request.body)
    assert api_request_body["geometry"] == {
        "type": "Point",
        "coordinates": otm_request_parameters["coordinates"],
    }

    properties_keys = ["max_cloud_cover", "min_off_nadir", "max_off_nadir"]

    if product == "assured":
        assert "signature" in api_request_body["properties"].keys()
        for key in properties_keys:
            assert not api_request_body["properties"].get(key)
    else:
        assert "signature" not in api_request_body["properties"].keys()
        for key in properties_keys:
            assert api_request_body["properties"][key] == otm_request_parameters[key]

    assert (
        api_request_body["properties"]["datetime"]
        == f"{otm_request_parameters['date_from'].isoformat()}/{otm_request_parameters['date_to'].isoformat()}"  # noqa: E501
    )


@mocketize(strict_mode=True)
def test_post_assured_order_without_signature(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
):
    otm_request_parameters["product"] = "assured"

    with raises(
        Exception,
        match="Orders with assured priority must also have a signature token.",
    ):
        client.otm_v2.create_order(**otm_request_parameters)


@mocketize(strict_mode=True)
@mark.parametrize(
    "per_page",
    (None, 10),
)
def test_list_tasking_orders(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
    per_page,
):
    contract_id = otm_request_parameters["contract_id"]
    per_page = per_page if not None else 25
    api_path = (
        API_PATH_ORDERS.replace("contract-id", str(contract_id)) + f"?{per_page=}"
    )

    Entry.single_register(
        "GET",
        client._gateway_url + api_path,
        status=200,
        body=dumps(otm_response),
    )

    response = client.otm_v2.list_orders(contract_id=contract_id, per_page=per_page)

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
def test_get_tasking_order(
    oauth_token_entry,
    client,
    otm_request_parameters,
    otm_response,
):
    order_id = uuid4()
    contract_id = otm_request_parameters["contract_id"]
    api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

    Entry.single_register(
        "GET",
        client._gateway_url + api_path + str(order_id),
        status=200,
        body=dumps(otm_response),
    )

    response = client.otm_v2.get_order(contract_id=contract_id, order_id=order_id)

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path + f"{order_id}"
    assert api_request.headers["Authorization"] == "Bearer mock-token"


@mocketize(strict_mode=True)
def test_post_search(
    oauth_token_entry,
    client,
    otm_request_parameters,
    search_response,
):
    contract_id = otm_request_parameters["contract_id"]
    api_path = "otm/v2/contract-id/search/".replace("contract-id", str(contract_id))

    Entry.single_register(
        "POST",
        client._gateway_url + api_path,
        status=200,
        body=dumps(search_response),
    )

    search_parameters = {
        "per_page": 100,
        "collections": ["feasibility"],
        "date_range": "2023-01-01T00:00:00/2023-01-31T11:59:59",
        "properties": {"min_gsd": 15, "max_gsd": 35, "status": "feasible"},
        "intersects": {},
        "sort_by": [{"field": "min_gsd", "direction": "asc"}],
    }

    response = client.otm_v2.search(contract_id=contract_id, **search_parameters)

    assert isinstance(response, dict)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/" + api_path
    assert api_request.headers["Authorization"] == "Bearer mock-token"



from datetime import datetime, timedelta
from importlib import import_module
from json import dumps
from typing import Dict, Optional
from urllib.parse import urljoin
from uuid import uuid4

from mocket.mockhttp import Entry
from pytest import fixture, mark, param

try:
    from requests import Session as RequestsSession
except ImportError:
    RequestsSession = None
try:
    from httpx import Client as HttpxClient
except ImportError:
    HttpxClient = None

from satellitevu.auth.cache import AbstractCache
from satellitevu.client import Client


@fixture(
    params=(
        param("UrllibClient"),
        param(
            "requests.RequestsSession",
            marks=[
                mark.skipif(RequestsSession is None, reason="requests is not installed")
            ],
        ),
        param(
            "httpx.HttpxClient",
            marks=[mark.skipif(HttpxClient is None, reason="httpx is not installed")],
        ),
    )
)
def http_client_class(request):
    full_path = f"satellitevu.http.{request.param}"
    module = import_module(full_path.rsplit(".", maxsplit=1)[0])
    return getattr(module, full_path.rsplit(".")[-1])


class MemoryCache(AbstractCache):
    cache: Dict[str, str]

    def __init__(self):
        super().__init__()
        self.cache = {}

    def save(self, client_id: str, value: str):
        self.cache[client_id] = value

    def load(self, client_id: str) -> Optional[str]:
        return self.cache.get(client_id)


@fixture
def client(memory_cache):
    return Client(client_id="mock-id", client_secret="mock-secret", cache=memory_cache)


@fixture
def memory_cache():
    return MemoryCache()


@fixture
def oauth_token_entry(client):
    return Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": "mock-token"}),
    )


@fixture
def redirect_response():
    return {"url": "https://image.test"}


@fixture
def order_details_response():
    return {
        "id": "528b0f77-5df1-4ed7-9224-502817170613",
        "type": "FeatureCollection",
        "features": [
            {
                "id": "8566d747-8833-44a7-a1c8-8d13e541f673",
                "properties": {
                    "item_id": "image",
                    "owned_by": "random",
                    "created_at": "2022-10-20T09:23:14.959823",
                    "updated_at": "2022-10-20T09:23:14.959823",
                },
            }
        ],
    }


@fixture
def otm_request_parameters():
    now = datetime.now().utcnow()
    return {
        "coordinates": [0, 0],
        "date_from": now,
        "date_to": (now + timedelta(hours=24)).utcnow(),
        "day_night_mode": "night",
        "max_cloud_cover": 100,
        "min_off_nadir": 20,
        "max_off_nadir": 30,
        "contract_id": str(uuid4()),
    }


@fixture
def otm_response(otm_request_parameters):
    return {
        "bbox": [0.0, 0.0, 0.0, 0.0],
        "type": "Feature",
        "geometry": {
            "type": "Point",
            "coordinates": otm_request_parameters["coordinates"],
        },
        "properties": {
            "datetime": (
                f"{otm_request_parameters['date_from']}/"
                f"{otm_request_parameters['date_to']}"
            ),
            "satvu:day_night_mode": otm_request_parameters["day_night_mode"],
            "max_cloud_cover": otm_request_parameters["max_cloud_cover"],
            "min_off_nadir": otm_request_parameters["min_off_nadir"],
            "max_off_nadir": otm_request_parameters["min_off_nadir"],
            "min_gsd": 3.5,
            "max_gsd": 6.8,
            "status": "feasible",
        },
        "id": str(uuid4()),
        "contract_id": str(otm_request_parameters["contract_id"]),
        "links": [],
    }


@fixture
def search_response(otm_request_parameters):
    return {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "geometry": {
                    "type": "Point",
                    "coordinates": otm_request_parameters["coordinates"],
                },
                "properties": {
                    "datetime": (
                        f"{otm_request_parameters['date_from']}/"
                        f"{otm_request_parameters['date_to']}"
                    ),
                    "status": "feasible",
                    "max_gsd": 6.8,
                    "min_gsd": 3.5,
                    "max_off_nadir": 45,
                    "min_off_nadir": 0,
                    "day_night_mode": "day",
                    "max_cloud_cover": None,
                    "created_at": "2023-10-06T08:56:26.760670",
                    "updated_at": "2023-10-06T08:56:28.686189",
                },
                "id": "57645c80-fda4-4116-9270-6f1b2a097000",
                "contract_id": "74841d4e-60d4-4e73-a691-2dddffa10d92",
                "collection": "feasibility",
            },
        ],
        "context": {"limit": 25, "matched": 1, "returned": 1},
        "links": [],
    }
